@@ -2,34 +2,36 @@ use bevy::ecs::prelude::*;
 use bevy::transform::prelude::*;
 use fnv::FnvHashMap;
 
-use heron_core::{PhysicMaterial, RigidBody, RotationConstraints, Velocity};
+use heron_core::{Damping, PhysicMaterial, RigidBody, RotationConstraints, Velocity};
 
 use crate::convert::{IntoBevy, IntoRapier};
 use crate::rapier::dynamics::{
     IslandManager, JointSet, RigidBodyBuilder, RigidBodyHandle, RigidBodySet, RigidBodyType,
 };
-use crate::rapier::geometry::{ColliderHandle, ColliderSet};
+use crate::rapier::geometry::ColliderSet;
 
 pub(crate) type HandleMap = FnvHashMap<Entity, RigidBodyHandle>;
 
 #[allow(clippy::type_complexity)]
 pub(crate) fn create(
-    mut commands: Commands<'_>,
+    mut commands: Commands<'_, '_>,
     mut bodies: ResMut<'_, RigidBodySet>,
     mut handles: ResMut<'_, HandleMap>,
     query: Query<
+        '_,
         '_,
         (
             Entity,
             &GlobalTransform,
             &RigidBody,
             Option<&Velocity>,
+            Option<&Damping>,
             Option<&RotationConstraints>,
         ),
-        Without<RigidBodyHandle>,
+        Without<super::RigidBodyHandle>,
     >,
 ) {
-    for (entity, transform, body, velocity, rotation_constraints) in query.iter() {
+    for (entity, transform, body, velocity, damping, rotation_constraints) in query.iter() {
         let mut builder = RigidBodyBuilder::new(body_status(*body))
             .user_data(entity.to_bits().into())
             .position((transform.translation, transform.rotation).into_rapier());
@@ -57,16 +59,22 @@ pub(crate) fn create(
                 .angvel(v.angular.into_rapier());
         }
 
+        if let Some(d) = damping {
+            builder = builder.linear_damping(d.linear).angular_damping(d.angular);
+        }
+
         let rigid_body_handle = bodies.insert(builder.build());
 
         handles.insert(entity, rigid_body_handle);
-        commands.entity(entity).insert(rigid_body_handle);
+        commands
+            .entity(entity)
+            .insert(super::RigidBodyHandle(rigid_body_handle));
     }
 }
 
 #[allow(clippy::too_many_arguments)]
 pub(crate) fn remove_invalids_after_components_removed(
-    mut commands: Commands<'_>,
+    mut commands: Commands<'_, '_>,
     mut handles: ResMut<'_, HandleMap>,
     mut bodies: ResMut<'_, RigidBodySet>,
     mut islands: ResMut<'_, IslandManager>,
@@ -75,6 +83,8 @@ pub(crate) fn remove_invalids_after_components_removed(
     bodies_removed: RemovedComponents<'_, RigidBody>,
     constraints_removed: RemovedComponents<'_, RotationConstraints>,
     materials_removed: RemovedComponents<'_, PhysicMaterial>,
+    rb_entities: Query<'_, '_, Entity, With<super::RigidBodyHandle>>,
+    collider_entities: Query<'_, '_, Entity, With<super::ColliderHandle>>,
 ) {
     bodies_removed
         .iter()
@@ -82,24 +92,35 @@ pub(crate) fn remove_invalids_after_components_removed(
         .chain(materials_removed.iter())
         .for_each(|entity| {
             if let Some(handle) = handles.remove(&entity) {
-                remove_collider_handles(&mut commands, &bodies, &colliders, handle);
+                remove_collider_handles(
+                    &mut commands,
+                    &collider_entities,
+                    &bodies,
+                    &colliders,
+                    handle,
+                );
                 bodies.remove(handle, &mut islands, &mut colliders, &mut joints);
-                commands.entity(entity).remove::<RigidBodyHandle>();
+                if rb_entities.get(entity).is_ok() {
+                    commands.entity(entity).remove::<super::RigidBodyHandle>();
+                }
             }
         });
 }
 
 #[allow(clippy::type_complexity)]
 pub(crate) fn remove_invalids_after_component_changed(
-    mut commands: Commands<'_>,
+    mut commands: Commands<'_, '_>,
     mut handles: ResMut<'_, HandleMap>,
     mut bodies: ResMut<'_, RigidBodySet>,
     mut islands: ResMut<'_, IslandManager>,
     mut colliders: ResMut<'_, ColliderSet>,
     mut joints: ResMut<'_, JointSet>,
+    collider_entities: Query<'_, '_, Entity, With<super::ColliderHandle>>,
+    rigidbody_entities: Query<'_, '_, Entity, With<super::RigidBodyHandle>>,
     changed: Query<
         '_,
-        (Entity, &RigidBodyHandle),
+        '_,
+        (Entity, &super::RigidBodyHandle),
         Or<(
             Changed<RigidBody>,
             Changed<RotationConstraints>,
@@ -108,16 +129,25 @@ pub(crate) fn remove_invalids_after_component_changed(
     >,
 ) {
     for (entity, handle) in changed.iter() {
-        remove_collider_handles(&mut commands, &bodies, &colliders, *handle);
-        bodies.remove(*handle, &mut islands, &mut colliders, &mut joints);
-        commands.entity(entity).remove::<RigidBodyHandle>();
+        remove_collider_handles(
+            &mut commands,
+            &collider_entities,
+            &bodies,
+            &colliders,
+            handle.0,
+        );
+        bodies.remove(handle.0, &mut islands, &mut colliders, &mut joints);
+        if rigidbody_entities.get(entity).is_ok() {
+            commands.entity(entity).remove::<super::RigidBodyHandle>();
+        }
         handles.remove(&entity);
     }
 }
 
 #[allow(clippy::manual_filter_map)]
 fn remove_collider_handles(
-    commands: &mut Commands<'_>,
+    commands: &mut Commands<'_, '_>,
+    entities: &Query<'_, '_, Entity, With<super::ColliderHandle>>,
     bodies: &RigidBodySet,
     colliders: &ColliderSet,
     handle: RigidBodyHandle,
@@ -131,17 +161,20 @@ fn remove_collider_handles(
             #[allow(clippy::cast_possible_truncation)]
             Entity::from_bits(it.user_data as u64)
         })
+        .filter(|e| entities.get(*e).is_ok())
         .for_each(|collider_entity| {
-            commands.entity(collider_entity).remove::<ColliderHandle>();
+            commands
+                .entity(collider_entity)
+                .remove::<super::ColliderHandle>();
         });
 }
 
 pub(crate) fn update_rapier_position(
     mut bodies: ResMut<'_, RigidBodySet>,
-    query: Query<'_, (&GlobalTransform, &RigidBodyHandle), Changed<GlobalTransform>>,
+    query: Query<'_, '_, (&GlobalTransform, &super::RigidBodyHandle), Changed<GlobalTransform>>,
 ) {
     for (transform, handle) in query.iter() {
-        if let Some(body) = bodies.get_mut(*handle) {
+        if let Some(body) = bodies.get_mut(handle.0) {
             let isometry = (transform.translation, transform.rotation).into_rapier();
             if body.is_kinematic() {
                 body.set_next_kinematic_position(isometry);
@@ -155,11 +188,12 @@ pub(crate) fn update_rapier_position(
 pub(crate) fn update_bevy_transform(
     bodies: Res<'_, RigidBodySet>,
     mut query: Query<
+        '_,
         '_,
         (
             Option<&mut Transform>,
             &mut GlobalTransform,
-            &RigidBodyHandle,
+            &super::RigidBodyHandle,
             Option<&RigidBody>,
         ),
     >,
@@ -169,7 +203,7 @@ pub(crate) fn update_bevy_transform(
             continue;
         }
 
-        let body = match bodies.get(*handle) {
+        let body = match bodies.get(handle.0) {
             None => continue,
             Some(body) => body,
         };
